@@ -1,7 +1,6 @@
 ; Goby: goby-ps.el
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
-;; Created: Aug 29, 2003
 
 ;;; Commentary:
 
@@ -16,8 +15,8 @@
 (defvar goby-ps-have-eps nil)
 
 (mapcar 'make-variable-buffer-local
-	(list 'goby-ps-eps-position
-	      'goby-ps-have-eps))
+	'(goby-ps-eps-position
+	  goby-ps-have-eps))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
