@@ -2,7 +2,7 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Aug  9, 2003
-;; Revised: Sep 20, 2007
+;; Revised: Apr 12, 2009
 
 ;;; Commentary:
 
@@ -10,10 +10,11 @@
 
 ;;; Code:
 
-(defconst goby-version "0.94")
+(defconst goby-version "1.0")
 
 (require 'goby-vars)
 (require 'goby-emacs)
+(require 'rect)
 
 (autoload 'goby-view-mode   "goby-view" nil t)
 (autoload 'goby-dump-screen "goby-view" nil t)
@@ -39,8 +40,7 @@
 
 (defvar goby-buffer-keymap nil)
 
-(mapcar 'make-variable-buffer-local
-	(list 'goby-buffer-keymap))
+(make-variable-buffer-local 'goby-buffer-keymap)
 
 (defvar goby-a-fonts (list goby-helvetica goby-courier goby-times))
 (defvar goby-j-fonts (list goby-gothic goby-mincho))
@@ -201,7 +201,7 @@ Minor mode for editing large TrueType fonts and images.
 \\{goby-mode-map}"
   (interactive "P")
   ;; All buffer local variables are killed after a major mode
-  ;; is called. 
+  ;; is called.
   (unless (eq major-mode goby-major-mode)
     (funcall goby-major-mode))
   (setq goby-mode (if (null arg)
@@ -362,7 +362,7 @@ Minor mode for editing large TrueType fonts and images.
       (unless ent
 	(setq ent (car undo))
 	(setq undo (cdr undo)))
-      (catch 'loop 
+      (catch 'loop
 	(while ent
 	  (if (stringp (car-safe ent)) (throw 'loop (setq found (car ent))))
 	  (setq ent (car undo))
@@ -387,8 +387,8 @@ Minor mode for editing large TrueType fonts and images.
       (goby-highlight-line (goby-face-get))
       (forward-line))))
 
-;; 
-;; 
+;;
+;;
 
 (defvar goby-highlight-ignore-command-list '(yank))
 
@@ -455,7 +455,20 @@ Minor mode for editing large TrueType fonts and images.
 (defun goby-face-set-color (frc color)
   (setcar (nthcdr 2 frc) color))
 
-(defun goby-face-region (beg end func &optional func2)
+(defun goby-face-region (beg end rectangle func &optional func2)
+  (if rectangle
+      (apply-on-rectangle 'goby-face-rectangle beg end func func2)
+    (goby-face-region1 beg end func func2)))
+
+(defun goby-face-rectangle (cbeg cend func &optional func2)
+  (let (beg end)
+    (move-to-column cend)
+    (setq end (point))
+    (move-to-column cbeg)
+    (setq beg (point))
+    (goby-face-region1 beg end func func2)))
+
+(defun goby-face-region1 (beg end func &optional func2)
   (let* ((after-change-functions nil)
 	 (aje (goby-face-get))
 	 (a-face (goby-get-aje-a aje))
@@ -489,18 +502,20 @@ Minor mode for editing large TrueType fonts and images.
 ;;; Ratio
 ;;;
 
-(defun goby-face-increase-ratio-region (beg end)
-  "Enlarge the characters in the region."
-  (interactive "r")
+(defun goby-face-increase-ratio-region (beg end &optional rectangle)
+  "Enlarge the characters in the region. 
+If called with '\\[universal-argument]', the target is the rectangle."
+  (interactive "r\nP")
   (goby-face-region
-   beg end 
+   beg end rectangle
    (lambda (frc) (goby-face-set-ratio frc (1+ (goby-face-get-ratio frc))))))
 
-(defun goby-face-decrease-ratio-region (beg end)
-  "Shrink the characters in the region."
-  (interactive "r")
+(defun goby-face-decrease-ratio-region (beg end &optional rectangle)
+  "Shrink the characters in the region.
+If called with '\\[universal-argument]', the target is the rectangle."
+  (interactive "r\nP")
   (goby-face-region
-   beg end 
+   beg end rectangle
    (lambda (frc) (goby-face-set-ratio frc (1- (goby-face-get-ratio frc))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -508,11 +523,12 @@ Minor mode for editing large TrueType fonts and images.
 ;;; Font
 ;;;
 
-(defun goby-face-next-font-region (beg end)
-  "Convert the characters in the region to the next font."
-  (interactive "r")
+(defun goby-face-next-font-region (beg end &optional rectangle)
+  "Convert the characters in the region to the next font.
+If called with '\\[universal-argument]', the target is the rectangle."
+  (interactive "r\nP")
   (goby-face-region
-   beg end
+   beg end rectangle
    (lambda (frc)
      (let* ((a-fontp (looking-at goby-ascii-regex))
 	    (base (if a-fontp goby-a-fonts-base goby-j-fonts-base))
@@ -527,11 +543,12 @@ Minor mode for editing large TrueType fonts and images.
 ;;; Color
 ;;;
 
-(defun goby-face-next-color-region (beg end)
-  "Convert the characters in the region to the next color."
-  (interactive "r")
+(defun goby-face-next-color-region (beg end &optional rectangle)
+  "Convert the characters in the region to the next color.
+If called with '\\[universal-argument]', the target is the rectangle."
+  (interactive "r\nP")
   (goby-face-region
-   beg end 
+   beg end rectangle
    (lambda (frc)
      (let* ((lim (length goby-tab-spec))
 	    (num (goby-face-get-color frc))) ;; xxx
@@ -539,12 +556,14 @@ Minor mode for editing large TrueType fonts and images.
        (if (= num lim) (setq num 0))
        (goby-face-set-color frc num)))))
 
-(defun goby-face-color-region (beg end)
-  (interactive "r")
+(defun goby-face-color-region (beg end &optional rectangle)
+  "Convert the characters in the region to the specified color.
+If called with '\\[universal-argument]', the target is the rectangle."
+  (interactive "r\nP")
   (let* ((color (completing-read "Color: " goby-favorite-color nil t))
 	 (n (goby-favarite-color-number color)))
     (goby-face-region
-     beg end 
+     beg end rectangle
      (lambda (frc) (goby-face-set-color frc n)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -567,8 +586,7 @@ Minor mode for editing large TrueType fonts and images.
     (goto-char beg)
     (while (re-search-forward goby-math-regex end t)
       (goby-face-region
-       (match-beginning 0)
-       (match-end 0)
+       (match-beginning 0) (match-end 0) nil
        (lambda (frc)
 	 (goby-face-set-family frc goby-m-fonts-base)))))
   (goby-face-math-set-prop beg end t))
@@ -581,11 +599,12 @@ Minor mode for editing large TrueType fonts and images.
   (let ((disp (get-text-property pos 'display)))
     (nth 1 (memq 'raise disp))))
 
-(defun goby-face-math-power-region (beg end)
-  "Shrink the characters in the region and raise them."
-  (interactive "r")
+(defun goby-face-math-power-region (beg end &optional rectangle)
+  "Shrink the characters in the region and raise them.
+If called with '\\[universal-argument]', the target is the rectangle."
+  (interactive "r\nP")
   (goby-face-region
-   beg end 
+   beg end rectangle
    (lambda (frc)
      (let* ((ratio (goby-face-get-ratio frc))
 	    raise)
@@ -606,11 +625,12 @@ Minor mode for editing large TrueType fonts and images.
    ((= num 5) 3)
    (t (- num 3))))
 
-(defun goby-face-math-aux-region (beg end)
-  "Shrink the characters in the region."
-  (interactive "r")
+(defun goby-face-math-aux-region (beg end &optional rectangle)
+  "Shrink the characters in the region.
+If called with '\\[universal-argument]', the target is the rectangle."
+  (interactive "r\nP")
   (goby-face-region
-   beg end 
+   beg end rectangle
    (lambda (frc)
      (goby-face-set-ratio frc (goby-face-math-magic (goby-face-get-ratio frc)))
      (goby-face-set-family frc goby-m-fonts-base))))
@@ -731,7 +751,7 @@ Minor mode for editing large TrueType fonts and images.
 
 (defun goby-create-image (file &optional scale)
   (let ((type (goby-image-type file))
-	topnm pixel-width data)
+	topnm pixel-width data image)
     (when (and (file-readable-p file)
 	       (or (null scale)
 		   (setq topnm (goby-get-topnm file))))
@@ -750,8 +770,9 @@ Minor mode for editing large TrueType fonts and images.
 	   (setq data (buffer-substring-no-properties (point-min) (point-max))))
 	  (setq type 'jpeg)))
       (if data
-	  (create-image data type 'data)
-	(create-image file type)))))
+	  (setq image (create-image data type 'data))
+	(setq image (create-image file type nil)))
+      (nconc image (list :mewfile file)))))
 
 (defun goby-visualize-image ()
   (goto-char (point-min))
@@ -1052,7 +1073,7 @@ Minor mode for editing large TrueType fonts and images.
 	(math  (goby-get-property1 'goby-math)))
     (if math (setq faces (cons (cons 'goby-math math) faces)))
     faces))
-    
+
 (defun goby-save-property (&optional to-buffer)
   (let ((props (goby-get-property))
 	(file (buffer-file-name))
@@ -1163,7 +1184,7 @@ Minor mode for editing large TrueType fonts and images.
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1172,7 +1193,7 @@ Minor mode for editing large TrueType fonts and images.
 ;; 3. Neither the name of the author nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
